edge_colours = {
    'increases': "limegreen",
    'directly_increases': "springgreen4",
    'decreases': "orangered",
    'directly_decreases': "red3",
    'rate_limiting_step_of': "lightslateblue",
    'regulates': "lightblue3",
    'causes_no_change': "yellow2",
    'positive_correlation': "blue"
}

node_colours = {
    'protein': "lightblue1",
    'complex': "khaki1",
    'component': "aquamarine",
    'rna': "goldenrod1",
    'gene': "lightslateblue",
    'activity': "palegreen3",
    'abundance': 'darksalmon',
    'pathology': "palegreen",
}
