"""Constant values for testing ebel_rest. Here we use the publicly available COVID-19 Knowledge
Graph login information."""

USER = "covid_user"
SERVER = "http://graphstore.scai.fraunhofer.de"
PASSWORD = "covid"
DATABASE = "covid"
